@@ -27,7 +27,7 @@ use proc_macro2::{Ident, Span};
 use proc_macro_crate::{crate_name, FoundCrate};
 use syn::spanned::Spanned;
 use syn::{Data, Field, Fields, DeriveInput, Error};
-
+use proc_macro2::TokenStream as TokenStream2;
 use crate::utils::is_lint_attribute;
 
 mod decode;
@@ -36,34 +36,35 @@ mod max_encoded_len;
 mod utils;
 mod trait_bounds;
 
-/// Include the `parity-scale-codec` crate under a known name (`_parity_scale_codec`).
-fn include_parity_scale_codec_crate() -> proc_macro2::TokenStream {
+/// Returns the identifier of the `parity-scale-codec` crate as used.
+///
+/// The identifier might change if the depending crate imported it
+/// using a custom package name.
+fn parity_scale_codec_ident() -> Result<TokenStream2, Error> {
+	static CRATE_NAME: &str = "parity-scale-codec";
+	fn root_import(name: &str) -> TokenStream2 {
+		let ident = Ident::new(name, Span::call_site());
+		quote!{ :: #ident }
+	}
 	// This "hack" is required for the tests.
-	if std::env::var("CARGO_PKG_NAME").unwrap() == "parity-scale-codec" {
-		quote!( extern crate parity_scale_codec as _parity_scale_codec; )
+	if std::env::var("CARGO_PKG_NAME").unwrap() == CRATE_NAME {
+		Ok(root_import("parity_scale_codec"))
 	} else {
-		match crate_name("parity-scale-codec") {
-			Ok(FoundCrate::Itself) => quote!( extern crate parity_scale_codec as _parity_scale_codec; ),
-			Ok(FoundCrate::Name(parity_codec_crate)) => {
-				let ident = Ident::new(&parity_codec_crate, Span::call_site());
-				quote!( extern crate #ident as _parity_scale_codec; )
-			},
-			Err(e) => Error::new(Span::call_site(), &e).to_compile_error(),
+		match crate_name(CRATE_NAME) {
+			Ok(FoundCrate::Itself) => {
+				Ok(quote! { crate })
+			}
+			Ok(FoundCrate::Name(custom_name)) => Ok(root_import(&custom_name)),
+			Err(e) => Err(Error::new(Span::call_site(), &e)),
 		}
 	}
 }
 
 /// Wraps the impl block in a "dummy const"
 fn wrap_with_dummy_const(input: DeriveInput, impl_block: proc_macro2::TokenStream) -> proc_macro::TokenStream {
-	let parity_codec_crate = include_parity_scale_codec_crate();
 	let attrs = input.attrs.into_iter().filter(is_lint_attribute);
-
 	let generated = quote! {
 		const _: () = {
-			#[allow(unknown_lints)]
-			#[cfg_attr(feature = "cargo-clippy", allow(useless_attribute))]
-			#[allow(rust_2018_idioms)]
-			#parity_codec_crate
 			#(#attrs)*
 			#impl_block
 		};
@@ -148,15 +149,23 @@ pub fn encode_derive(input: proc_macro::TokenStream) -> proc_macro::TokenStream
 		return e.to_compile_error().into();
 	}
 
+	let crate_ident = match crate::parity_scale_codec_ident() {
+		Ok(crate_ident) => crate_ident,
+		Err(error) => {
+			return error.into_compile_error().into()
+		}
+	};
+
 	if let Some(custom_bound) = utils::custom_encode_trait_bound(&input.attrs) {
 		input.generics.make_where_clause().predicates.extend(custom_bound);
 	} else if let Err(e) = trait_bounds::add(
 		&input.ident,
 		&mut input.generics,
 		&input.data,
-		parse_quote!(_parity_scale_codec::Encode),
+		parse_quote!(#crate_ident::Encode),
 		None,
 		utils::has_dumb_trait_bound(&input.attrs),
+		&crate_ident,
 	) {
 		return e.to_compile_error().into();
 	}
@@ -164,14 +173,14 @@ pub fn encode_derive(input: proc_macro::TokenStream) -> proc_macro::TokenStream
 	let name = &input.ident;
 	let (impl_generics, ty_generics, where_clause) = input.generics.split_for_impl();
 
-	let encode_impl = encode::quote(&input.data, name);
+	let encode_impl = encode::quote(&input.data, name, &crate_ident);
 
 	let impl_block = quote! {
-		impl #impl_generics _parity_scale_codec::Encode for #name #ty_generics #where_clause {
+		impl #impl_generics #crate_ident::Encode for #name #ty_generics #where_clause {
 			#encode_impl
 		}
 
-		impl #impl_generics _parity_scale_codec::EncodeLike for #name #ty_generics #where_clause {}
+		impl #impl_generics #crate_ident::EncodeLike for #name #ty_generics #where_clause {}
 	};
 
 	wrap_with_dummy_const(input, impl_block)
@@ -191,15 +200,23 @@ pub fn decode_derive(input: proc_macro::TokenStream) -> proc_macro::TokenStream
 		return e.to_compile_error().into();
 	}
 
+	let crate_ident = match crate::parity_scale_codec_ident() {
+		Ok(crate_ident) => crate_ident,
+		Err(error) => {
+			return error.into_compile_error().into()
+		}
+	};
+
 	if let Some(custom_bound) = utils::custom_decode_trait_bound(&input.attrs) {
 		input.generics.make_where_clause().predicates.extend(custom_bound);
 	} else if let Err(e) = trait_bounds::add(
 		&input.ident,
 		&mut input.generics,
 		&input.data,
-		parse_quote!(_parity_scale_codec::Decode),
+		parse_quote!(#crate_ident::Decode),
 		Some(parse_quote!(Default)),
 		utils::has_dumb_trait_bound(&input.attrs),
+		&crate_ident,
 	) {
 		return e.to_compile_error().into();
 	}
@@ -209,13 +226,13 @@ pub fn decode_derive(input: proc_macro::TokenStream) -> proc_macro::TokenStream
 	let ty_gen_turbofish = ty_generics.as_turbofish();
 
 	let input_ = quote!(__codec_input_edqy);
-	let decoding = decode::quote(&input.data, name, &quote!(#ty_gen_turbofish), &input_);
+	let decoding = decode::quote(&input.data, name, &quote!(#ty_gen_turbofish), &input_, &crate_ident);
 
 	let impl_block = quote! {
-		impl #impl_generics _parity_scale_codec::Decode for #name #ty_generics #where_clause {
-			fn decode<__CodecInputEdqy: _parity_scale_codec::Input>(
+		impl #impl_generics #crate_ident::Decode for #name #ty_generics #where_clause {
+			fn decode<__CodecInputEdqy: #crate_ident::Input>(
 				#input_: &mut __CodecInputEdqy
-			) -> ::core::result::Result<Self, _parity_scale_codec::Error> {
+			) -> ::core::result::Result<Self, #crate_ident::Error> {
 				#decoding
 			}
 		}
@@ -249,13 +266,21 @@ pub fn compact_as_derive(input: proc_macro::TokenStream) -> proc_macro::TokenStr
 		return e.to_compile_error().into();
 	}
 
+	let crate_ident = match crate::parity_scale_codec_ident() {
+		Ok(crate_ident) => crate_ident,
+		Err(error) => {
+			return error.into_compile_error().into()
+		}
+	};
+
 	if let Err(e) = trait_bounds::add(
 		&input.ident,
 		&mut input.generics,
 		&input.data,
-		parse_quote!(_parity_scale_codec::CompactAs),
+		parse_quote!(#crate_ident::CompactAs),
 		None,
 		utils::has_dumb_trait_bound(&input.attrs),
+		&crate_ident,
 	) {
 		return e.to_compile_error().into();
 	}
@@ -311,22 +336,22 @@ pub fn compact_as_derive(input: proc_macro::TokenStream) -> proc_macro::TokenStr
 	};
 
 	let impl_block = quote! {
-		impl #impl_generics _parity_scale_codec::CompactAs for #name #ty_generics #where_clause {
+		impl #impl_generics #crate_ident::CompactAs for #name #ty_generics #where_clause {
 			type As = #inner_ty;
 			fn encode_as(&self) -> &#inner_ty {
 				#inner_field
 			}
 			fn decode_from(x: #inner_ty)
-				-> ::core::result::Result<#name #ty_generics, _parity_scale_codec::Error>
+				-> ::core::result::Result<#name #ty_generics, #crate_ident::Error>
 			{
 				::core::result::Result::Ok(#constructor)
 			}
 		}
 
-		impl #impl_generics From<_parity_scale_codec::Compact<#name #ty_generics>>
+		impl #impl_generics From<#crate_ident::Compact<#name #ty_generics>>
 			for #name #ty_generics #where_clause
 		{
-			fn from(x: _parity_scale_codec::Compact<#name #ty_generics>) -> #name #ty_generics {
+			fn from(x: #crate_ident::Compact<#name #ty_generics>) -> #name #ty_generics {
 				x.0
 			}
 		}
